/***************************************************************/
/* Prof. Dr. Carsten Vogt                                      */
/* FH Koeln, Fak. 07 / Nachrichtentechnik                      */
/* http://www.nt.fh-koeln.de/vogt                              */
/*                                                             */
/* Das Programm zeigt am Beispiel von Bankkonten die Benutzung */
/* des Klassenkonzepts, insbesondere                           */
/* - Definition von Klassen mit klassen- und objektbezogenen   */
/*   Attributen / Methoden sowie oeffentlichen und privaten    */
/*   Attributen,                                               */
/* - Erzeugung und Loeschung von Objekten und                  */
/* - Vererbung mit Nutzung und Ueberschreiben von Methoden der */
/*   Superklasse.                                              */
/***************************************************************/

import java.io.*;

/****************************************************************

   Die Klasse 'Konto' ist die Basisklasse fuer Konten.
   Sie enthaelt als Attribute
   - die Anzahl existierender Konten (klassenbezogen) und
   - den Kontostand (objektbezogen).
   Sie ermoeglicht mit ihren Methoden
   - das Erzeugen eines Kontos mit Ersteinzahlung,
   - die Abfrage der Anzahl der existierenden Konten,
   - die Abfrage des Kontostands,
   - das Einzahlen und
   - das Abheben (sofern ausreichend Deckung vorhanden).
   
*****************************************************************/

class Konto {

 /* 'anzahlKonten' (klassenbezogenes Attribut)
    gibt die Anzahl existierender Konten an */

 static private int anzahlKonten = 0;

 /* 'stand' gibt den aktuellen Kontostand an */

 private double stand;

 /* Konstruktor 'Konto()': setzt Anfangsstand auf
    'initBetrag' und erhoeht 'anzahlKonten' um 1 */

 Konto(double stand) {
  this.stand = stand;
  anzahlKonten++;
 } 

 /* Destruktor 'finalize()': senkt 'anzahlKonten' um 1 */

 protected void finalize() throws Throwable {
  anzahlKonten--;
  System.out.println("Konto geloescht");
 } 

 /* 'getAnzahlkonten()' liefert Anzahl der Konten (klassenbezogene Methode) */

 static public int getAnzahlKonten() {
  return anzahlKonten;
 }

 /* 'abfragen()' liefert aktuellen Kontostand */

 public double abfragen() {
  return stand;
 }

 /* 'einzahlen()' erhoeht Kontostand um Einzahlungsbetrag 'betrag' */

 public void einzahlen(double betrag) {
  stand += betrag;
 }

 /* 'abheben()' senkt Kontostand um Abhebebetrag 'betrag', wenn Kontostand
    dafuer ausreicht.
    liefert ausgezahlten Betrag oder -1, wenn Abheben nicht erfolgreich. */

 public double abheben(double betrag) {
  if (betrag<=stand)
   { stand -= betrag;
     return betrag; }
  else return -1;  
 }

}

/****************************************************************

   Die Klasse 'GeschuetztesKonto' erweitert die Klasse
   'Konto' um Kontrollmechanismen.
   Sie enthaelt als Attribute
   - eine SuperPIN (klassenbezogen), die gebraucht wird, um die
     PIN eines Konto aendern zu koennen (aehnlich der Legitimation
     eines Bankangestellten) und
   - eine PIN (objektbezogen) fuer das Abheben.
   Sie ermglicht mit ihren Methoden
   - das Erzeugen eines Kontos mit Ersteinzahlung,
   - das Setzen der PIN eines Kontos (sofern SuperPIN
     korrekt angegeben) und
   - das Abheben (sofern ausreichend Deckung vorhanden
     und PIN korrekt angegeben).   
      
****************************************************************/

class GeschuetztesKonto extends Konto {
 
 /* superPin (klassenbezogene Variable):
    Die PIN eines Kontos kann nur geaendert werden,
    wenn 'superPin' korrekt angeben wird. */

 static private int superPin = 4711;

 /* pin: Abheben ist nur bei Angabe der korrekten PIN moeglich */ 

 private int pin;

 /* Konstruktor 'GeschuetztesKonto()' */

 GeschuetztesKonto(double stand, int pin) {
  super(stand);
  this.pin = pin;
 }  

 /* 'pinSetzen()' setzt die PIN eines Kontos auf 'p'.
    Dazu muss ueber 'sup' die korrekte SuperPIN eingegeben werden.
    Liefert 0 bei Erfolg und -1 sonst. */ 

 public int pinSetzen(int sup, int p){
  if (sup==superPin) {
   pin = p;
   return 0;
  }
  else return -1;
 }

 /* 'abheben(betrag,p)' versucht, den Betrag 'betrag' vom Konto
    abzuheben. Voraussetzung ist die Angabe der korrekten
    PIN ueber 'p' und ein ausreichender Kontostand.
    Liefert den abgehobenen Betrag bei Erfolg und -1 sonst. */

 public double abheben(double betrag, int p) {
  if (pin!=p) return -1;
  return super.abheben(betrag);
 }

 /* 'abheben(betrag)' ueberschreibt die entsprechende Methode der
    Superklasse Konto. Die abheben()-Methode der Klasse Konto ist
    damit bei einem Objekt der Klasse GeschuetztesKonto nicht mehr
    aufrufbar, so dass ohne PIN kein Geld abgehoben werden kann. */

 public double abheben(double betrag) {
  System.out.println("Abheben nur mit PIN!");
  return -1;
 }

}

/*****************************************************

   Die Klasse 'BankkontenAll' ist das Hauptprogramm.
      
******************************************************/

public class BankkontenAll {

 public static void main(String args[]) throws IOException {

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

  Konto konto1ungesch;

  GeschuetztesKonto konto2gesch;

  char dummy;

  double betr;

  int pi, spi;    

  /*********** Test von nicht geschuetzten Konten ***********/

  System.out.println();
  System.out.println("Eroeffnung eines nicht geschuetzten Kontos 'Konto 1'");
  System.out.println();
  System.out.print("Bitte gewuenschten Anfangsbetrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  konto1ungesch = new Konto(betr); 
  System.out.println();

  System.out.println("Anzahl existierender Konten: " + Konto.getAnzahlKonten());
  System.out.println("Kontostand von Konto 1:      " + konto1ungesch.abfragen());
  System.out.println();

  System.out.print("Einzahlen - bitte Betrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  konto1ungesch.einzahlen(betr);
  System.out.println("Kontostand von Konto 1:    " + konto1ungesch.abfragen());
  System.out.println();
  
  System.out.print("Abheben - bitte Betrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  if (konto1ungesch.abheben(betr)>=0.0)
    System.out.println("Abheben erfolgreich");
   else System.out.println("Abheben fehlgeschlagen");
  System.out.println("Kontostand von Konto 1:    " + konto1ungesch.abfragen());
  System.out.println();
 
  System.out.print("Nochmal abheben - bitte Betrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  if (konto1ungesch.abheben(betr)>=0.0)
    System.out.println("Abheben erfolgreich");
   else System.out.println("Abheben fehlgeschlagen");
  System.out.println("Kontostand von Konto 1:    " + konto1ungesch.abfragen());
  System.out.println();
 
  konto1ungesch = null;
  System.gc(); /* Garbage Collector, um Objekt tatsachlich zu loeschen */ 
  System.out.println("Anzahl existierender Konten: " + Konto.getAnzahlKonten());

  /*********** Test von geschuetzten Konten ***********/

  System.out.println();
  System.out.println("Eroeffnung eines geschuetzten Kontos 'Konto 2'");
  System.out.println();
  System.out.print("Bitte gewuenschten Anfangsbetrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  System.out.print("Bitte gewuenschte PIN eingeben: ");
  pi = Integer.parseInt(in.readLine());
  konto2gesch = new GeschuetztesKonto(betr,pi); 

  System.out.println();
  System.out.println("Anzahl existierender Konten: " + Konto.getAnzahlKonten());
  System.out.println("Kontostand von Konto 2:      " + konto2gesch.abfragen());
  System.out.println();

  System.out.print("Abheben - bitte Betrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  System.out.print("Bitte PIN eingeben: ");
  pi = Integer.parseInt(in.readLine());
  if (konto2gesch.abheben(betr,pi)>=0.0)
    System.out.println("Abheben erfolgreich");
   else System.out.println("Abheben fehlgeschlagen");
  System.out.println("Kontostand von Konto 2:    " + konto2gesch.abfragen());
  System.out.println();
 
  System.out.print("PIN aendern - bitte Super-PIN eingeben: ");
  spi = Integer.parseInt(in.readLine());
  System.out.print("Bitte gewuenschte neue PIN eingeben: ");
  pi = Integer.parseInt(in.readLine());
  if (konto2gesch.pinSetzen(spi,pi)==0)
    System.out.println("Aenderung erfolgreich");
   else System.out.println("Aenderung fehlgeschlagen");
  System.out.println();
 
  System.out.print("Abheben - bitte Betrag eingeben: ");
  betr = Double.parseDouble(in.readLine());
  System.out.print("Bitte PIN eingeben: ");
  pi = Integer.parseInt(in.readLine());
  if (konto2gesch.abheben(betr,pi)>=0.0)
    System.out.println("Abheben erfolgreich");
   else System.out.println("Abheben fehlgeschlagen");
  System.out.println("Kontostand von Konto 2:    " + konto2gesch.abfragen());
  System.out.println();

 }

}
